declare const isPromise: <T>(value: unknown) => value is Promise<T>;
declare const isPromiseLike: <T>(value: unknown) => value is PromiseLike<T>;

/**
 * In browser, roaring library initialization is done asynchronously, this method returns true after roaring library WASM is initialized.
 * The library cannot be used until this function returns true.
 * You can await initialization with roaringLibraryInitialize function that returns a promise.
 * @returns true if roaring library WASM is initialized.
 */
declare const roaringLibraryIsReady: () => boolean;
/**
 * In browser, roaring library initialization is done asynchronously, this method returns a promise that resolves when roaring library WASM is initialized.
 * The library cannot be used until this promise is resolved.
 * @returns a promise that resolves when roaring library WASM is initialized.
 * @example
 * await roaringLibraryInitialize();
 * const bitmap = new RoaringBitmap32([123]);
 * console.log(bitmap.toArray()); // [123]
 */
declare const roaringLibraryInitialize: () => Promise<void>;

/**
 * Array of unsigned 32 bit integers allocted directly in roaring library WASM memory.
 * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
 */
declare class RoaringUint32Array implements Iterable<number> {
  /**
   * The type of typed array used by this class.
   * For RoaringUint32Array is Uint32Array.
   */
  static readonly TypedArray: typeof Uint32Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint32Array is always 4
   */
  static readonly BYTES_PER_ELEMENT: 4;
  /**
   * The type of typed array used by this class.
   * For RoaringUint32Array is Uint32Array.
   */
  get TypedArray(): typeof Uint32Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint32Array is always 4
   */
  get BYTES_PER_ELEMENT(): 4;
  /**
   * The ArrayBuffer instance referenced by the array.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned buffer for short periods of time.
   */
  get buffer(): ArrayBuffer;
  /**
   * Returns true if this object was deallocated.
   */
  get isDisposed(): boolean;
  /**
   * The length in bytes of the array.
   * For RoaringUint32Array it is equal to this.length * 4
   */
  get byteLength(): number;
  /**
   * The full WASM heap in hich this array is allocated.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned array for short periods of time.
   */
  get heap(): Uint32Array;
  /**
   * The offset in bytes of the array (the location of the first byte in WASM memory).
   */
  readonly byteOffset: number;
  /**
   * Number of elements allocated in this array.
   */
  readonly length: number;
  /**
   * Allocates an array in the roaring WASM heap.
   *
   * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
   *
   * If the parameter is a number, it creates a new uninitialized array of the given length.
   * If the parameter is an Iterable, it creates a copy of the given iterable.
   *
   * @param lengthOrArray - Length of the array to allocate or the array to copy
   */
  constructor(lengthOrArray: number | Iterable<number>, _pointer?: number);
  /**
   * Frees the allocated memory.
   * Is safe to call this method more than once.
   *
   * @returns True if memory gets freed during this call, false if not.
   */
  dispose(): boolean;
  /**
   * Throws an error if the memory was freed.
   */
  throwIfDisposed(): void | never;
  /**
   * Writes the given array at the specified position
   * @param array - A typed or untyped array of values to set.
   * @param offset - The index in the current array at which the values are to be written.
   */
  set(array: Iterable<number>, offset?: number): this;
  /**
   * Gets a new JS typed array instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new typed array that shares the memory with this array.
   */
  asTypedArray(): Uint32Array;
  /**
   * Gets a new NodeJS Buffer instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new instance of NodeJS Buffer
   */
  asNodeBuffer(): Buffer;
  /**
   * Copies the content of this buffer to a typed array.
   * The returned array is garbage collected and don't need to be disposed manually.
   *
   * @returns A new typed array that contains a copy of this buffer
   */
  toTypedArray(): Uint32Array;
  /**
   * Copies the content of this buffer to a NodeJS Buffer.
   * The returned buffer is garbage collected and don't need to be disposed manually.
   *
   * @returns A new instance of NodeJS Buffer that contains a copy of this buffer
   */
  toNodeBuffer(): Buffer;
  /**
   * Copies the content of this typed array into a new JS array of numbers and returns it.
   *
   * @returns A new array.
   */
  toArray(): number[];
  /**
   * Copies the content of this typed array into a new JS Set<number> and returns it.
   *
   * @returns A new array.
   */
  toSet(): Set<number>;
  /**
   * Returns a string representation of an array.
   */
  toString(): string;
  /**
   * Iterator that iterates through all values in the array.
   */
  [Symbol.iterator](): IterableIterator<number>;
}

/**
 * Array of bytes allocted directly in roaring library WASM memory.
 * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
 */
declare class RoaringUint8Array implements Iterable<number> {
  /**
   * The type of typed array used by this class.
   * For RoaringUint8Array is Uint8Array.
   */
  static readonly TypedArray: typeof Uint8Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint8Array is always 1
   */
  static readonly BYTES_PER_ELEMENT: 1;
  /**
   * The type of typed array used by this class.
   * For RoaringUint8Array is Uint8Array.
   */
  get TypedArray(): typeof Uint8Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint8Array is always 1
   */
  get BYTES_PER_ELEMENT(): 1;
  /**
   * The ArrayBuffer instance referenced by the array.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned buffer for short periods of time.
   */
  get buffer(): ArrayBuffer;
  /**
   * Returns true if this object was deallocated.
   */
  get isDisposed(): boolean;
  /**
   * The length in bytes of the array.
   * For RoaringUint8Array it is equal to this.length
   */
  get byteLength(): number;
  /**
   * The full WASM heap in hich this array is allocated.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned array for short periods of time.
   */
  get heap(): Uint8Array;
  /**
   * The offset in bytes of the array (the location of the first byte in WASM memory).
   */
  readonly byteOffset: number;
  /**
   * Number of elements allocated in this array.
   */
  readonly length: number;
  /**
   * Allocates an array in the roaring WASM heap.
   *
   * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
   *
   * If the parameter is a number, it creates a new uninitialized array of the given length.
   * If the parameter is an Iterable, it creates a copy of the given iterable.
   *
   * @param lengthOrArray - Length of the array to allocate or the array to copy
   */
  constructor(lengthOrArray: number | Iterable<number>, _pointer?: number);
  /**
   * Frees the allocated memory.
   * Is safe to call this method more than once.
   * @returns True if memory gets freed during this call, false if not.
   */
  dispose(): boolean;
  /**
   * Throws an error if the memory was freed.
   */
  throwIfDisposed(): void | never;
  /**
   * Writes the given array at the specified position
   * @param array - A typed or untyped array of values to set.
   * @param offset - The index in the current array at which the values are to be written.
   */
  set(array: Iterable<number>, offset?: number): this;
  /**
   * Gets a new JS typed array instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new typed array that shares the memory with this array.
   */
  asTypedArray(): Uint8Array;
  /**
   * Gets a new NodeJS Buffer instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new instance of NodeJS Buffer
   */
  asNodeBuffer(): Buffer;
  /**
   * Copies the content of this buffer to a typed array.
   * The returned array is garbage collected and don't need to be disposed manually.
   *
   * @returns A new typed array that contains a copy of this buffer
   */
  toTypedArray(): Uint8Array;
  /**
   * Copies the content of this buffer to a NodeJS Buffer.
   * The returned buffer is garbage collected and don't need to be disposed manually.
   *
   * @returns A new instance of NodeJS Buffer that contains a copy of this buffer
   */
  toNodeBuffer(): Buffer;
  /**
   * Copies the content of this typed array into a standard JS array of numbers and returns it.
   *
   * @returns A new array.
   */
  toArray(): number[];
  /**
   * Returns a string representation of an array.
   */
  toString(): string;
  /**
   * Iterator that iterates through all values in the array.
   */
  [Symbol.iterator](): IterableIterator<number>;
}

/**
 * A Roaring Bitmap that supports 32 bit unsigned integers.
 *
 * The roaring bitmap allocates in WASM memory, remember to dispose
 * the RoaringBitmap32 when not needed anymore to release WASM memory.
 *
 */
declare class RoaringBitmap32 {
  private _ptr;
  /**
   * Creates a new roaring bitmap adding the specified values.
   *
   * The roaring bitmap allocates in WASM memory, remember to dispose
   * the RoaringBitmap32 when not needed anymore to release WASM memory.
   * @param values - The values to add
   */
  constructor(values?: RoaringUint32Array | Iterable<number>);
  /**
   * Creates a new roaring bitmap deserializing it from a buffer
   *
   * The roaring bitmap allocates in WASM memory, remember to dispose
   * the RoaringBitmap32 when not needed anymore to release WASM memory.
   *
   * @param buffer - The buffer to deserialize
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   * @returns The reulting bitmap. Remember to dispose the instance when finished using it.
   */
  static deserialize(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): RoaringBitmap32;
  /**
   * Utility function that serializes an array of uint32 to a new NodeJS buffer.
   * The returned buffer is automatically garbage collected.
   *
   * @param values - The values to serialize, a bitmap or an iterable of uint32 values.
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The NodeJS buffer containing the serialized data.
   */
  static serializeArrayToNewBuffer(
    values: RoaringUint32Array | Iterable<number>,
    portable?: boolean
  ): Buffer;
  /**
   * Utility function that deserializes a RoaringBitmap32 serialized in a buffer to an Array<number> of values.
   * The array can be very big, be careful when you use this function.
   *
   * @param buffer - The buffer to deserialize.
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   * @returns All the values in the bitmap.
   */
  static deserializeToArray(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): number[];
  /**
   * Utility function that deserializes a RoaringBitmap32 serialized in a buffer to a Set<number> of values.
   * The array can be very big, be careful when you use this function.
   *
   * @param buffer - The buffer to deserialize.
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   * @returns All the values in the bitmap.
   */
  static deserializeToSet(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): Set<number>;
  /**
   * Returns true if this instance was disposed.
   */
  get isDisposed(): boolean;
  /**
   * Disposes this object freeing all WASM memory associated to it.
   * Is safe to call this method more than once.
   */
  dispose(): boolean;
  /**
   * Throws an exception if this object was disposed before.
   */
  throwIfDisposed(): void | never;
  /**
   * Get the cardinality of the bitmap (number of elements).
   */
  cardinality(): number;
  /**
   * Returns true if the bitmap has no elements.
   */
  isEmpty(): boolean;
  /**
   * Adds a 32 bit unsigned integer value.
   * Values are unique, this function does nothing if the value already exists.
   */
  add(value: number): void;
  /**
   * Adds a 32 bit unsigned integer value checking if the bitmap changes.
   * Use add() if you don't need to know if something changed.
   * Values are unique, this function does nothing and returns false if the value already exists.
   *
   * @param value - 32 bit unsigned integer to add in the set.
   * @returns True if the bitmap changed, false if not.
   */
  addChecked(value: number): boolean;
  /**
   * Adds multiple values.
   * Using this is faster than calling add() multiple times.
   * Inserting ordered or partially ordered arrays is faster.
   *
   * @param values - The values to add.
   */
  addMany(values: RoaringUint32Array | Iterable<number>): void;
  /**
   * Removes a value from the set.
   * If the value does not exists, this function does nothing.
   *
   * @param value - The value to remove.
   */
  remove(value: number): void;
  /**
   * Removes a value from the set checking if the bitmap changes.
   * Use remove() if you don't need to know if something changed.
   * If the value does not exists, this function does nothing and returns false.
   *
   * @param value - 32 bit unsigned integer to remove from the set.
   * @returns True if the bitmap changed, false if not.
   */
  removeChecked(value: number): boolean;
  /**
   * Gets the maximum value stored in the bitmap.
   * If the bitmap is empty, returns 0.
   *
   * @returns The maximum 32 bit unsigned integer or 0 if empty.
   */
  maximum(): number;
  /**
   * Gets the minimum value stored in the bitmap.
   * If the bitmap is empty, returns 0xFFFFFFFF
   *
   * @returns The minimum 32 bit unsigned integer or 0xFFFFFFFF if empty.
   */
  minimum(): number;
  /**
   * Checks whether the given value is contained in the set.
   *
   * @param value - The value to look for.
   * @returns True if value exists in the set, false if not.
   */
  contains(value: number): boolean;
  /**
   * Returns true if the bitmap is subset of the other.
   *
   * @param other - the other bitmap
   * @returns true if the bitmap is subset of the other.
   */
  isSubset(other: RoaringBitmap32): boolean;
  /**
   * Returns true if this bitmap is strict subset of the other.
   *
   * @param other - The other bitmap
   * @returns True if this bitmap is a strict subset of other
   */
  isStrictSubset(other: RoaringBitmap32): boolean;
  /**
   * Converts the bitmap to an array.
   * The array may be very big, use this function with caution.
   * The returned RoaringUint32Array is allocated in WASM memory and not garbage collected,
   * it need to be freed manually calling dispose().
   *
   * @returns The RoaringUint32Array. Remember to manually dispose to free the memory.
   */
  toRoaringUint32Array(): RoaringUint32Array;
  /**
   * Converts the bitmap to a JS array.
   * The resulting array may be very big, use this function with caution.
   *
   * @returns The array containing all values in the bitmap.
   */
  toArray(): number[];
  /**
   * Converts the bitmap to a JS Set<number>.
   * The resulting set may be very big, use this function with caution.
   *
   * @returns The set containing all values in the bitmap.
   */
  toSet(): Set<number>;
  /**
   * Converts the bitmap to a JS Uint32Array.
   * The resulting array may be very big, use this function with caution.
   *
   * @returns The array containing all values in the bitmap.
   */
  toUint32Array(): Uint32Array;
  /**
   * Checks wether two roaring bitmap contains the same data.
   *
   * @param other - The other bitmap.
   * @returns True if the bitmaps contains the same data, false if not.
   */
  equals(other: RoaringBitmap32): boolean;
  /**
   * Negates in place the bitmap within a specified interval.
   * Areas outside the range are passed through unchanged.
   *
   * @param start - Range start.
   * @param end - Range end.
   */
  flipRange(start: number, end: number): void;
  /**
   * Optimizes the bitmap releasing unused memory and compressing containers.
   * Returns true if something changed.
   *
   * @returns True if something changed.
   */
  optimize(): boolean;
  /**
   * If the size of the roaring bitmap is strictly greater than rank, then
   * this function returns the element of given rank.
   * Otherwise, it returns NaN.
   *
   * @param rank - Element rank
   * @returns element or NaN
   */
  select(rank: number): number;
  /**
   * Computes the size of the intersection between two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the intersection between two bitmaps.
   */
  andCardinality(other: RoaringBitmap32): number;
  /**
   * Computes the size of the union of two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the union of two bitmaps.
   */
  orCardinality(other: RoaringBitmap32): number;
  /**
   * Computes the size of the difference (andnot) of two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the difference (andnot) of two bitmaps.
   */
  andNotCardinality(other: RoaringBitmap32): number;
  /**
   * Computes the size of the symmetric difference (xor) between two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the symmetric difference (xor) of two bitmaps.
   */
  xorCardinality(other: RoaringBitmap32): number;
  /**
   * Intersects this bitmap with another.
   * Removes the elements from this bitmap that don't exists in the other.
   * Stores the result in this bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  andInPlace(other: RoaringBitmap32): void;
  /**
   * Adds the element of the other bitmap into this bitmap.
   * Stores the result in this bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  orInPlace(other: RoaringBitmap32): void;
  /**
   * Computes the difference between two bitmaps.
   * Stores the result in this bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  xorInPlace(other: RoaringBitmap32): void;
  /**
   * Compute the difference between this and the provided bitmap,
   * writing the result in the current bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  andNotInPlace(other: RoaringBitmap32): void;
  /**
   * Returns the number of integers that are smaller or equal to the given value.
   *
   * @param value - The value to rank
   * @returns The number of values smaller than the given value
   */
  rank(value: number): number;
  /**
   * Check whether the two bitmaps intersect (have at least one element in common).
   *
   * @param other - The other bitmap.
   * @returns True if the two bitmaps intersects, false if not.
   */
  intersects(other: RoaringBitmap32): boolean;
  /**
   * Computes the Jaccard index between two bitmaps.
   * (Also known as the Tanimoto distance, or the Jaccard similarity coefficient)
   * See https://en.wikipedia.org/wiki/Jaccard_index
   *
   * The Jaccard index is undefined if both bitmaps are empty.
   *
   * @returns The Jaccard index
   */
  jaccardIndex(other: RoaringBitmap32): number;
  /**
   * How many bytes are required to serialize this bitmap.
   *
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   */
  getSerializationSizeInBytes(portable?: boolean): number;
  /**
   * Serializes a bitmap to a byte buffer allocated in WASM memory.
   *
   * The returned RoaringUint8Array is allocated in WASM memory and not garbage collected,
   * it need to be freed manually calling dispose().
   *
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The RoaringUint8Array. Remember to manually dispose to free the memory.
   */
  serializeToRoaringUint8Array(portable?: boolean): RoaringUint8Array;
  /**
   * Serializes a bitmap to a typed Uint8Array.
   * The returned array is automatically garbage collected and there is no need to be disposed manually.
   *
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The Uint8Array that contains the serialized bitmap
   */
  serializeToUint8Array(portable?: boolean): Uint8Array;
  /**
   * Serializes a bitmap to a NodeJS buffer.
   * The returned buffer is automatically garbage collected and there is no need to be disposed manually.
   *
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The NodeJS Buffer that contains the serialized bitmap
   */
  serializeToNodeBuffer(portable?: boolean): Buffer;
  /**
   * Reads a bitmap from a serialized version.
   * Throws an error if deserialization failed.
   *
   * @param buffer - The buffer to deserialize
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   */
  deserialize(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): void;
}

export {
  RoaringBitmap32,
  RoaringUint32Array,
  RoaringUint8Array,
  isPromise,
  isPromiseLike,
  roaringLibraryInitialize,
  roaringLibraryIsReady,
};
